@@ -14,13 +14,24 @@ from sqlalchemy import sql, orm
 from sqlalchemy import and_
 from sqlalchemy import func 
 from sqlalchemy.sql.expression import desc
-from sqlalchemy.exc import OperationalError
+from sqlalchemy.exc import OperationalError, InvalidRequestError
 
 from plone.memoize import instance, ram
 
 from sc.social.viewcounter.pageview import session
 from sc.social.viewcounter.pageview import PageView
 
+from Acquisition import aq_inner
+from plone.app.portlets.cache import get_language
+
+def hourly_cache(fun, self, *args, **kw):
+    lang = get_language(aq_inner(self.context), self.request)
+    return "%s-%s" % (lang, time() // (60 * 60))
+
+def daily_cache(fun, self, *args, **kw):
+    lang = get_language(aq_inner(self.context), self.request)
+    return "%s-%s" % (lang, time() // (60 * 60 * 24))
+
 import logging
 logger = logging.getLogger('sc.social.viewcounter:')
 
@@ -42,7 +53,7 @@ class PageViewBV(BrowserView):
         session = self.session
         try:
             session.connection()
-        except OperationalError:
+        except (OperationalError,InvalidRequestError):
             logger.info('SQL: Cannot connect to database')
             session = None
         context = self.context
@@ -85,7 +96,7 @@ class Reports(BrowserView):
         super(Reports, self).__init__(context, request)
     
     
-    @ram.cache(lambda *args: time() // (60 * 60))
+    @ram.cache(hourly_cache)
     def _reportPageViews(self,dateRange=None,portal_type='',path='',**kw):
         logger.debug('Query sql.')
         session = self.session
@@ -122,7 +133,7 @@ class Reports(BrowserView):
         return dictByUIDs
         
     @property
-    @ram.cache(lambda *args: time() // (60 * 60))
+    @ram.cache(hourly_cache)
     def lastHour(self):
         now = datetime.now()
         delta = timedelta(0,3600)
@@ -144,7 +155,7 @@ class Reports(BrowserView):
         return (start,end)
     
     @property
-    @ram.cache(lambda *args: time() // (60 * 60))
+    @ram.cache(hourly_cache)
     def lastDay(self):
         now = datetime.now()
         delta = timedelta(1,0)
@@ -166,7 +177,7 @@ class Reports(BrowserView):
         return (start,end)
     
     @property
-    @ram.cache(lambda *args: time() // (60 * 60 * 24))
+    @ram.cache(daily_cache)
     def lastWeek(self):
         now = datetime.now()
         weekday = now.weekday()
@@ -192,7 +203,7 @@ class Reports(BrowserView):
     
     
     @property
-    @ram.cache(lambda *args: time() // (60 * 60 * 24))
+    @ram.cache(daily_cache)
     def lastMonth(self):
         now = datetime.now()
         day = now.day
@@ -286,4 +297,4 @@ class Reports(BrowserView):
     def viewsAllTime(self,**kw):
         # If we really want everything, let's do it :-)
         range = None
-        return self._reportPageViews(range, **self.cleanKw(kw))
\ No newline at end of file
+        return self._reportPageViews(range, **self.cleanKw(kw))
